/*
  STATUS OK
  Version EckNowa 1.0*/
  int ino_version = 6;
  long ino_date = 200120;
  
  #define READ_EEPROM false					// set true if reading CAL values from Eeprom ; else false
  #include "includes_main.h"				// include all libs
  
  /*
  Kompiliert unter Arduino 1.8.10

  1. Verwendung Wire.h siehe Datei "main_includes"
  
  2. Verwendung I2Cdev.h siehe Datei "main_includes"
 
  3. Verwendete Hardware, die bei mir funktioniert
  3.1 LCD KeypadShield
  3.2 Arduino UNO R3
  3.3 ADXL345 BreakoutBoard, GY-521

  
    ###### Anpassungen U.Nowakowski / strgaltdel Jan 2019:
  - alle Lib-Einbindungen nach "main_includes.h" ausgelagert
  - kein sensorspezifisches Coding in main-app
  - Bibliothek "imu.h" mit Methoden zum Auslesen von IMU-Breakout boards; Änderungen von Sensor HW muessen nur dort vorgenommen werden
	-  Initialisierung Sensor:					.init()
	-  Auslesen Raw Werte:						.getRawAccValues()
	-	 Auslesen Winkel: 						.getAngleValues()
	-	 Auslesen Winkel (nach TP Filterung):	.getfilteredAngleValues()
	-	 Auslesen Winkel (nach DEMA Filterung):	.getDEMAfilteredAngleValues()
  - imu.h mit Unterstützung GY521 (höhere Auflösung / 16bit)
  - Kalibrierwerte in Struktur definiert
  - Mittelwertbildung der Lesewerte über Parameter "lcdRefreshRate"
  - kleinere Formatierungs Anpassungen wg float Darstellung
  
  
  
  Eckkehard (bis Ende 2019):
  -Fixes: Help Screen kommt nun zuverlaessiger, switch/case statt if/else
  -Fixes: SerialDebug Schnittstelle, Ausgabe "Initialize Serial Port..." wird nun ausgegeben
  -Verbesserung: Button RIGHT zeigt HELP Screen an
  -Verbesserung: Ausgabe der Version und Datum
  -Erweitert: Wert CHORD wird mit Button SELECT im EEPROM gespeichert
  -Maximalwert CHORD 999mm begrenzt
  -Ausgabe der Prefixe ANG und THR reduzuiert, damit konfigurierter Wert CHRORD angezeigt werden kann
  -Sensor2 (rechter Sensor): Orientierung angepasst, Sensor um 180Grad gegenüber Sensor1 gedreht eingesetzt wird
  -Code Review, Vereinfachung der Programmstruktur, LCD Ausgabe und Aktion in einem switch/case/.../default vereint
  
 
*/





AngleSens IMUsensor = AngleSens();							// init class IMUsensor
Isensor IMU_0, IMU_1;										// init struct IMU  (Angle values sensor:0 & 1)


/************************************ LCD refresh interval ***********************************/
unsigned long lcdRefreshRate = 1400;     // lcd refresh rate (milliseconds)
unsigned long startMillis = 0;  		// used to calculate lcd refresh
unsigned long actualMillis  = 0;		// used to calculate lcd refresh


/*******************************************************************************************/
//Fuer das LCD KeyPad-Shield
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

/*******************************************************************************************/
// define some address to write data
int addr = 0;
/*******************************************************************************************/
// define some values used by the LCD panel and KeypadShield buttons
int lcd_key     = 0;
int adc_key_in  = 0;
/*******************************************************************************************/
int toggle = LOW;  // variable for toggling between THROW and SECANT
int chord;         // chord depth [mm]
/*******************************************************************************************/
boolean serialDebug = true; // usefull using adruino uno, does not work on other arduino boards
/************************************ DEFINED VARIABLES ************************************/
// define some values used by angle calculation
//int x1, y1, z1;               // raw values from sensor (-255 ... 0 ... +255)
//int x2, y2, z2;               // raw values from sensor (-255 ... 0 ... +255)



float avSum1, avSum2, avCounter;				// sum up read values for building average
float angle1;
float angle2;
int throw1;
int throw2;
int secant1;
int secant2;
int up1;                        // 1 -> up, 2 -> down
int up2;                        // 1 -> up, 2 -> down
float relative1;                  // for zeroing
float relative2;                  // for zeroing
float rad1;
float rad2;



/************************************ DEFINED CONSTANTS ************************************/
#define btnRIGHT  0
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5
/************************************ Grapics for display **********************************/
uint8_t degree[8]  = {140, 146, 146, 140, 128, 128, 128, 128};      // °
uint8_t arrowup[8]  = {132, 142, 132, 132, 132, 128, 128, 128};     // Arrow up
uint8_t arrowdown[8]  = {128, 128, 128, 132, 132, 132, 142, 132,};  // Arrow down

/*******************************************************************************************/
/************************************ DEFINED FUNCTIONS ************************************/
/*******************************************************************************************/
int read_LCD_buttonsFunction()
{
  adc_key_in = analogRead(0);      // read the value from the sensor / KeypadShield
  // the values depends on KeypadShield, use
  if (adc_key_in > 1000) return btnNONE;
  if (adc_key_in < 50)   return btnRIGHT;
  if (adc_key_in < 150)  return btnUP;
  if (adc_key_in < 300)  return btnDOWN  ;
  if (adc_key_in < 500)  return btnLEFT;
  if (adc_key_in < 700)  return btnSELECT;
  return btnNONE;
}

void show_UsageFunction(boolean los)
{
switch (los) {
  case true:
  {
    lcd.home ();
    lcd.print("RC - METER      ");
    lcd.setCursor (0, 1);
    lcd.print("ANGLE AND THROW ");
    delay(3000);
    lcd.clear ();
    lcd.print("RC - METER");
    lcd.setCursor (0, 1);
    lcd.print("  >>  OR  <<");
    delay(1500);
    lcd.clear ();
    lcd.print("RC - METER");
    lcd.setCursor (0, 1);
    lcd.print("ANGLE AND SECANT");
    delay(3000);

    lcd.clear ();
    lcd.print("to ZERO, press:");
    lcd.setCursor (0, 1);
    lcd.print("Key SELECT");
    delay(3000);

    lcd.clear ();
    lcd.print("set CORD depth..");
    delay(1500);
    lcd.setCursor (0, 0);
    lcd.print(".in [mm], press:");
    lcd.setCursor (0, 1);
    lcd.print("Key UP or DOWN");
    delay(3000);

    lcd.clear ();
    lcd.print("choose THROW or.");
    delay(1500);
    lcd.setCursor (0, 0);
    lcd.print("..SECANT, press:");
    lcd.setCursor (0, 1);
    lcd.print("Key LEFT");
    delay(3000);

    lcd.clear ();
    lcd.print("to RESET, press:");
    lcd.setCursor (0, 1);
    lcd.print("Key RST");
    delay(3000);

    lcd.clear ();
    lcd.print("for HELP screen");
    lcd.setCursor (0, 1);
    lcd.print("press: Key RIGHT");
    delay(3000);
    lcd.clear ();
    lcd.print("Version: ");
    lcd.print(ino_version);
    lcd.setCursor (0, 1);
    lcd.print("Date:    ");
    lcd.print(ino_date);
    delay(3000);
  }
   break;
  default:
  {
    lcd.home ();
    lcd.print("RC - METER      ");
    lcd.setCursor (0, 1);
    lcd.print("ANGLE AND THROW ");
    delay(2000);
    lcd.clear ();
    lcd.print("for HELP screen");
    lcd.setCursor (0, 1);
    lcd.print("press: Key RIGHT");
    delay(2000);
  }
  break;
}
}

/*******************************************************************************************/
/************************************** FUNCTION SETUP *************************************/
/*******************************************************************************************/
void setup() {
  Serial.begin(9600);
  Serial.println("Initialize IMU");
  // Initialize both IMU Sensors 
  IMUsensor.init(0);			//init IMU sensor:0
  IMUsensor.init(1);			//init IMU sensor:1
  

  // power on LCD
  lcd.begin (16, 2);            // for 16 x 2 LCD module

  lcd.createChar(0, degree);
  lcd.createChar(1, arrowup);
  lcd.createChar(2, arrowdown);
/*******************************************************************************************/
// START SCREEN
/*******************************************************************************************/
  show_UsageFunction(false);
/********************** Restore value of CHORD from last save ********************************/
  chord =  EEPROM.read(addr);
/********************** DEBUG output to serial port monitor **********************************/
  if (serialDebug) {
   while (!Serial) {
   ; // wait for serial port to connect. Needed for native USB
  }

    Serial.print("Initialize Serial Port...");
    delay(1000);
    Serial.println("OK");
    delay(1000);
  };
}


/*******************************************************************************************/
/**************************************** FUNCTION MAIN LOOP *******************************/
/*******************************************************************************************/
void loop() {



  char dummytext[18]; 			// textarray for lcd line
  actualMillis = millis();
 
 
  //IMU_0 = IMUsensor.getDEMAfilteredAngleValues(0);		// get DEMA smoothed angle values from sensor:0
  //IMU_1 = IMUsensor.getDEMAfilteredAngleValues(1);		// get DEMA smoothed angle values from sensor:1
  
  //IMU_0 = IMUsensor.getfilteredAngleValues(0);			// get LP smoothed angle values from sensor:0
  //IMU_1 = IMUsensor.getfilteredAngleValues(1);			// get LP smoothed angle values from sensor:1
  
  IMU_0 = IMUsensor.getAngleValues(0);						// get unsmoothed angle values from sensor:0
  IMU_1 = IMUsensor.getAngleValues(1);						// get unsmoothed angle values from sensor:1
 
  
  // and now fill variables to built average over n readings
  avCounter++;
  avSum1 += IMU_0.pitch;   
  avSum2 += IMU_1.pitch;    
  
  // check if new values should be displayed and refresh values
  if(actualMillis-startMillis >= lcdRefreshRate){
	
	angle1 = avSum1/avCounter;		// refresh angle values
	angle2 = avSum2/avCounter;
	
	// reset loop
	avCounter = 0;
    startMillis = millis();
	avSum1 = 0;
	avSum2 = 0;
  }
  
  if ((angle1 - relative1) > 0) {
    up1 = 1;                     //arrow ↑↑↑
  }
  else {
    up1 = 2;                     //arrow ↓↓↓
  }
  if ((angle2 - relative2) > 0) {
    up2 = 1;                     //arrow ↑↑↑
  }
  else {
    up2 = 2;                     //arrow ↓↓↓
  }

/*******************************************************************************************/
  rad1 = abs(angle1 - relative1) * 0.017453;
  rad2 = abs(angle2 - relative2) * 0.017453;

  throw1 = int(chord * sin(rad1));
  throw2 = int(chord * sin(rad2));

  secant1 = int(2 * chord * sin(rad1 / 2));
  secant2 = int(2 * chord * sin(rad2 / 2));

/*******************************************************************************************/
  lcd_key = read_LCD_buttonsFunction();

  switch (lcd_key)               // depending on which button was pushed, we perform an action
  {
    case btnSELECT:
      {
        relative1 = angle1;
        relative2 = angle2;
        EEPROM.update(addr, chord);

        lcd.clear();
        lcd.setCursor (0, 0);           // go to start of 2nd line
        lcd.print("zeroized...");
        lcd.setCursor (0, 1);           // go to start of 2nd line
        lcd.print("CHORD saved...");
        
        if (serialDebug) {Serial.println(lcd_key);} else {}
        break;
      }

    case btnLEFT:
      {
        lcd.clear();
        lcd.setCursor (0, 0);           // go to start of 2nd line

        if (toggle == HIGH) {
          toggle = LOW;
          lcd.print("TRHOW...");          
          if (serialDebug) {Serial.print(lcd_key); Serial.print(" THROW "); Serial.println(toggle); } else {}
        }
        else {
          toggle = HIGH;
          lcd.print("SECANT...");
          if (serialDebug) {Serial.print(lcd_key); Serial.print(" SECANT "); Serial.println(toggle); } else {}
        }
        delay (300);
        break;
      }

    case  btnUP: {
        if  (chord < 999) {
          chord = chord + 1;
        }
        else{}

        lcd.clear();
        lcd.setCursor (0, 0);           // go to start of 1st line
        lcd.print("CHORD DEPTH:");
        lcd.setCursor (0, 1);           // go to start of 2nd line
        lcd.print(chord); lcd.print(" mm");

        if (serialDebug) {Serial.print(lcd_key); Serial.print(" CHORD DEPTH:"); Serial.print(chord); Serial.println("mm");} else {}
        break;
      }

    case btnDOWN: {
        if  (chord > 1) {
          chord = chord - 1;
        }
        else{}

        lcd.clear();
        lcd.setCursor (0, 0);           // go to start of 1st line
        lcd.print("CHORD DEPTH:");
        lcd.setCursor (0, 1);           // go to start of 2nd line
        lcd.print(chord); lcd.print(" mm");

        if (serialDebug) { Serial.print(lcd_key); Serial.print(" CHORD DEPTH:"); Serial.print(chord); Serial.println("mm"); } else {}
        break;
      }

    case btnRIGHT:
      {
        if (serialDebug) {Serial.println(lcd_key);} else {}
        show_UsageFunction(true);
        break;
      }

    default: 
      {
        lcd.clear();

        lcd.setCursor (0, 0);         // go to start of 1st line
        lcd.print("A:");
        
        if ((angle1 - relative1) == 0) {
		  lcd.setCursor (4, 0);
          lcd.print(" 0");
          lcd.setCursor (14, 0);
          lcd.write(byte(0));
        }
        else {
		lcd.setCursor (1, 0); 
		dtostrf(abs(angle1 - relative1),5,1,dummytext);
        lcd.print(dummytext);
		lcd.write(byte(up1));
         // lcd.setCursor (14, 0);
         // lcd.write(byte(0));
        }

        lcd.setCursor (8, 0);
        if ((angle2 - relative2) == 0) {
          lcd.print(" 0");
          lcd.setCursor (13, 0);
          lcd.write(byte(0));
        }
        else {
		  dtostrf(abs(angle2 - relative2),4,1,dummytext);
          lcd.print(dummytext);
		  lcd.write(byte(up2));
         //lcd.setCursor (13, 0);
         //lcd.write(byte(0));
        }
 
          lcd.setCursor (14, 0);			// chord 
          lcd.print(chord);
          lcd.setCursor (14, 1);
          lcd.print("mm");
 
        lcd.setCursor (0, 1);          // go to start of 2nd line
        if (toggle == HIGH) {
          lcd.print("S");


          lcd.setCursor (1, 1);
          if ((secant1) == 0) {
            lcd.print("  0");
          }
          else {
            //lcd.print(secant1);
			dtostrf(secant1,3,0,dummytext);
            lcd.print(dummytext);
          }

          lcd.setCursor (7, 1);
          if ((secant2) == 0) {
            lcd.print("  0");
          }
          else {
            //lcd.print(secant2);
			dtostrf(secant2,3,0,dummytext);
            lcd.print(dummytext);
          }
        }
        else {
          lcd.print("T");


          lcd.setCursor (2, 1);
          if ((throw1) == 0) {
            lcd.print(" 0");
          }
          else {
			lcd.setCursor (1, 1); 
			dtostrf(throw1,3,0,dummytext);
            lcd.print(dummytext);
          }

          lcd.setCursor (7, 1);
          if ((throw2) == 0) {
            lcd.print("   0");
          }
          else {
			
			dtostrf(throw2,3,0,dummytext);
            lcd.print(dummytext);
          }
        }
        if (serialDebug) {Serial.println(lcd_key);} else {}
        break;
      }
  }
  delay(300);
}
  
